const _ = require("lodash");
const ko = require("knockout");
const moment = require("moment");

const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const {Wizard} = require("cattura-ui-library/src/lib/ui/wizard");
const ScheduledEvent = require("cattura-ui-library/src/lib/scheduledEvent");
const MediaPackage = require("cattura-ui-library/src/lib/mediaPackage");

const DEFAULT_OPTIONS = {
    recorders: [],
    momentFormatter: moment => moment.toString(),
    momentGenerator: moment,
    relativizeStoragePath: path => path,
    previewRecording: () => {},
    recorder: ko.observable()
};

function makeDialog (options)
{
    options = _({})
        .extend(DEFAULT_OPTIONS)
        .extend(options)
        .value();
    
    if (ko.isObservable(options.recorder))
    {
        options.recorder((ko.unwrap(options.recorders) || [])[0]);
    }
    
    const relativizeStoragePath = options.relativizeStoragePath;
    const previewRecording      = options.previewRecording;
    
    const intro = require("./step-0-intro");
    const details = require("./step-1-details")(options);
    const datetime = require("./step-2-date-time")({
        momentGenerator: options.momentGenerator
    });
    const assets = require("./step-3-assets")(options);
    const summary = require("./step-4-summary")({
        momentFormatter: options.momentFormatter,
        dateTime: datetime.dateTime
    });

    const {title, template, tags, appendDateToTitle} = details;
    const {dateTime, duration, recur, recurrenceStartDateTime, recurrenceEndDate, days} = datetime;

    /* jshint -W079 */
    const confirm = require("cattura-ui-library/src/modals/confirm");

    let isReadOnly = ko.observable(false);
    let readOnlyReason = ko.observable("");
    let eventId = ko.observable();
    let eventGroupId = ko.observable();
    
    // Raw event object returned from the API.
    // This doesn't exist for new events and does not change when the
    // event is modified. It's intended for external scheduler plugins
    // to be able to pull and display additional data associated to an
    // external event.
    let eventObject = ko.observable();

    // The recorder the currently loaded event belongs to.
    // For multi-recorder instances (i.e. Command Center).
    // All operations done on existing events (update, delete) use this instead
    // of the user-selected recorder observable.
    // This is null for new events.
    let parentRecorder = ko.observable();

    // For external events
    let externalSchedulerId = ko.observable();
    let externalSchedulerName = ko.observable();
    let externalEventUrl = ko.observable();
    const externalScheduler = ko.computed(() => {
        if (!externalSchedulerId())
        {
            return null;
        }
        
        return _(ko.unwrap((parentRecorder() || {}).externalSchedulers) || [])
            .find(plugin => plugin.id === externalSchedulerId());
    });
    
    // For completed recordings
    let mediaPackage = ko.observable();

    // For failed recordings
    let errorMessage = ko.observable();
    
    function extendMediaPackage (mediaPackage)
    {
        if (relativizeStoragePath)
        {
            mediaPackage.root = relativizeStoragePath(mediaPackage.root);
        }
        
        return mediaPackage;
    }

    function determineReadOnlyReason (event)
    {
        readOnlyReason("unknown");

        if (event.sourceName)
        {
            readOnlyReason("external");
            externalSchedulerId(event.sourceID);
            externalSchedulerName(event.sourceName);
            externalEventUrl(event.externalEventURL);
        }

        if (event.mediaPackageID)
        {
            readOnlyReason("complete");
            parentRecorder().apiv1.capture.getMediaPackage({
                id: event.mediaPackageID
            }).then(result => {
                mediaPackage(extendMediaPackage(new MediaPackage(result)));
            });
        }

        if (event.failedToStart())
        {
            readOnlyReason("failedToStart");
        }

        if (event.errorMessage)
        {
            readOnlyReason("error");
            errorMessage(event.errorMessage);
        }    
    }

    return Dialog({
        open (event, recorder)
        {
            this.reset();
            if (event && "startTime" in event)
            {
                eventId(event.id);
                eventObject(event);

                // Determine if this event belongs to a recorder
                // (if we're in multi-recorder mode)
                if (event.recorder)
                {
                    recorder = event.recorder;
                }
                else if (event.unit)
                {
                    recorder = event.unit;
                }
            }
            else
            {
                // Recorder passed in as first argument?
                if (!recorder && event && event.templates)
                {
                    recorder = event;
                }
                
                event = null;
            }
            
            // Map passed-in recorder object to an object we know about.
            // This object was either passed in as an argument or attached 
            // to the event object.
            const recorders = ko.unwrap(options.recorders) || [];
            if (recorder)
            {
                recorder = _(recorders).find(recorderObject => recorder === recorderObject ||
                                             recorderObject.id === recorder.id ||
                                             recorderObject.url === recorder.url);
            }
            // It is an error to pass in events with no associated recorders in multi-recorder mode.
            else if (recorders.length > 1)
            {
                throw new Error("addEditEvent modal is in multi-recorder mode but no recorder is associated with event");
            }
            // If in single-recorder mode then just default to the first one available
            else
            {
                recorder = recorders[0];
            }
            
            if (ko.isObservable(options.recorder))
            {
                options.recorder(recorder);
            }
            
            if (event)
            {
                // Keep track of recorder that this event belongs to
                parentRecorder(recorder);
                
                if (event.isReadOnly())
                {
                    isReadOnly(true);
                    determineReadOnlyReason(event);
                }

                let promise = Promise.resolve(event);
                if (event.groupID)
                {
                    eventGroupId(event.groupID);
                    promise = promise.then(() => {
                        return recorder.apiv1.scheduler.getEventGroup({
                            id: event.groupID
                        });
                    }).then((group) => {
                        return _({
                            group: group
                        }).extend(event).value();
                    });
                }

                promise.then((eventWithGroup) => {
                    _(this.wizard).each(step => {
                        if ("completed" in step)
                        {
                            step.completed(true);
                        }

                        if ("populate" in step)
                        {
                            step.populate(eventWithGroup);
                        }
                    });
                    this.show();
                    this.wizard.selectedStep(4);
                });                
            }
            else
            {
                parentRecorder(null);
                this.show();
            }
        },

        isReadOnly: isReadOnly,

        recordingEvent: {
            id: eventId,
            groupId: eventGroupId,
            title: title,
            template: template,
            tags: tags,
            dateTime: dateTime,
            duration: duration,
            assets: assets.assets,
            recur: recur,
            recurrenceEndDate: recurrenceEndDate,
            days: ko.computed(() => {
                return _(days).toPairs().map((pair, i) => {
                    return [i, { day: pair[0], selected: pair[1]() }];
                }).filter(pair => pair[1].selected).fromPairs().mapValues(value => value.day).value();
            }),
            externalSchedulerName: externalSchedulerName,
            externalScheduler: externalScheduler,
            externalEventUrl: externalEventUrl,
            readOnlyReason: readOnlyReason,
            mediaPackage: mediaPackage,
            errorMessage: errorMessage,
            recorder: parentRecorder,
            eventObject: eventObject,
            toArgs ()
            {
                return {
                    id: this.id(),
                    presentationTitle: this.title(),
                    template: this.template().id,
                    startTime: this.dateTime().valueOf(),
                    duration: this.duration().asMinutes(),
                    recurrenceEnd: this.recurrenceEndDate().valueOf(),
                    tags: (this.tags() || []).join(","),
                    days: _(this.days()).keys().join(","),
                    assets: this.assets()
                };
            },
            viewCompletedRecording ()
            {
                previewRecording.open(mediaPackage());
            }
        },

        reset ()
        {
            eventId("");
            eventGroupId("");
            externalSchedulerId("");
            externalSchedulerName("");
            externalEventUrl("");
            isReadOnly(false);
            readOnlyReason("");
            mediaPackage(null);
            eventObject(null);
            errorMessage("");
            this.wizard.reset();

        },

        wizard: Wizard({
            intro: intro,
            details: details,
            datetime: datetime,
            assets: assets,
            summary: summary
        }),

        deleteEvent ()
        {
            if (!this.recordingEvent.id())
            {
                return;
            }

            confirm.open(`Delete scheduled event ${this.recordingEvent.title()}?`).then(() => {
                return parentRecorder().apiv1.scheduler.deleteEvent({id: this.recordingEvent.id()}).then(retval => {
                    this.dispatchEvent({type: "eventdeleted", event: ScheduledEvent(retval)});
                    this.hide();
                }).catch(error => this.dispatchEvent({type: "error", message: error.message, error: error}));
            });
        },

        deleteRecurring ()
        {
            if (!this.recordingEvent.groupId())
            {
                return;
            }

            confirm.open(`Delete scheduled recurring event ${this.recordingEvent.title()}?`).then(() => {
                return parentRecorder().apiv1.scheduler.deleteEventGroup({id: this.recordingEvent.groupId()}).then(retval => {
                    this.dispatchEvent({type: "eventdeleted", event: ScheduledEvent(retval[0].event)});
                    this.hide();
                }).catch(error => this.dispatchEvent({type: "error", message: error.message, error: error}));
            });
        },

        // Create New Single or Recurring Event, or Update Single Event
        save ()
        {
            const recorder = this.recordingEvent.id() ? parentRecorder()
                                                      : ko.unwrap(options.recorder);
            
            // Creating a new recurring event will implicitly create all its events.
            if (this.recordingEvent.recur() && !this.recordingEvent.id())
            {
                return recorder.apiv1.scheduler.createEventGroup(this.recordingEvent.toArgs()).then(retval => {
                    this.dispatchEvent({type: "recurrencesaved", events: retval});
                    this.hide();
                }).catch(error => this.dispatchEvent({type: "error", message: error.message, error: error}));
            }

            return recorder.apiv1.scheduler[this.recordingEvent.id() ? 'updateEvent' : 'createEvent'](this.recordingEvent.toArgs()).then(retval => {
                this.dispatchEvent({type: "eventsaved", event: ScheduledEvent(retval)});
                this.hide();
            }).catch(error => this.dispatchEvent({type: "error", message: error.message, error: error}));
        },

        // Update Recurring Event
        saveRecurring ()
        {
            // Updating an existing recurring event will cause its child events to be changed accordingly.
            let args = this.recordingEvent.toArgs();
            args.id = this.recordingEvent.groupId();
            const recorder = args.id ? parentRecorder()
                                     : ko.unwrap(options.recorder);

            if (args.id)
            {
                args.startTime = recurrenceStartDateTime().valueOf();
            }

            return ko.unwrap(options.recorder).apiv1.scheduler.updateEventGroup(args).then(retval => {
                this.dispatchEvent({type: "recurrencesaved", events: retval});
                this.hide();
            }).catch(error => this.dispatchEvent({type: "error", message: error.message, error: error}));
        }
    }, {disableBodyScroll: false, easilyCloseable: false, reparent: true });
}

module.exports = makeDialog;