const _ = require("lodash");
const ko = require("knockout");
const moment = require("moment");

const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const selectable = require("cattura-ui-library/src/lib/ui/selectable");

const Exclusion = require("../../tab/exclusion.js");

const {makeMoment} = require("./time");

let exclusionId = ko.observable();
let label = ko.observable();
let startDate = ko.observable().extend({
    required: true
});
let endDate = ko.observable();
let repeat = ko.observable();

/*
 * Recurrence start/end dates in the UI are at midnight UTC, since the moment-recur
 * library seems to depend on that (it fails or hangs outright if they aren't).
 * However, we don't want to submit a midnight-UTC date to the backend, because it will
 * be interpreted under the system timezone and may result in an incorrect date.
 * To get around this, we instead create a new moment under the system timezone
 * and set the correct date on it.
 *
 * moment-timezone is buggy in general, see https://github.com/c-trimm/moment-recur/issues/85
 */
function makeDateMoment (date)
{
    return makeMoment().set({
        date: date.date(),
        month: date.month(),
        year: date.year()
    });
}

function makeDialog (plugin)
{
    return Dialog({
        id: exclusionId,

        fields: ko.validatedObservable({
            label: label,
            startDate: startDate,
            endDate: endDate,
            repeat: repeat,
        }),

        open (exclusion)
        {
            this.reset();

            if (exclusion instanceof Exclusion)
            {
                this.id(exclusion.id);
                this.fields().label(exclusion.label);
                this.fields().startDate(makeDateMoment(exclusion.startDate));

                if (exclusion.startDate.valueOf() != exclusion.endDate.valueOf())
                {
                    this.fields().endDate(makeDateMoment(exclusion.endDate));
                }

                this.fields().repeat(exclusion.recurrence ? exclusion.recurrence.frequency : "");
            }

            this.show();
        },

        reset ()
        {
            this.id(null);
            this.fields().label("");
            this.fields().startDate(moment());
            this.fields().endDate("mm/dd/yyyy");
            this.fields().repeat("");
        },

        save ()
        {
            if (!this.fields.isValid())
            {
                this.dispatchEvent({type: "error", message: "Form is not valid."});
                return;
            }

            let args = {
                label: this.fields().label() || "",
                startDate: this.fields().startDate().valueOf(),
                endDate: this.fields().startDate().valueOf()
            };

            if (this.fields().endDate() && !isNaN(this.fields().endDate().valueOf()))
            {
                args.endDate = this.fields().endDate().valueOf();
            }

            if (this.fields().repeat())
            {
                args.recurrence = `FREQ=${this.fields().repeat()}`;
            }

            let request;
            if (this.id())
            {
                request = plugin.services.updateExclusion(_({
                    id: this.id()
                }).extend(args).value());
            }
            else
            {
                request = plugin.services.addExclusion(args);
            }

            request.then(retval => {
                this.dispatchEvent({type: "exclusionsaved", exclusion: new Exclusion(retval)});
                this.hide();
            }).catch(error => this.dispatchEvent({type: "error", message: error.message, error: error}));
        }
    }, {
        template: require("./content.pug")
    });
}

module.exports = makeDialog;
