const _ = require("lodash");
const moment = require("moment");
//require("moment-recur");

/*
 * Exclusions in the UI use UTC moments, because moment-recur won't work with
 * anything else.
 * See https://github.com/c-trimm/moment-recur/issues/85 for other issues with
 * moment-recur.
 */
function localDateToMoment (localDate)
{
    return moment.utc({
        day: localDate.dayOfMonth,
        month: localDate.monthValue - 1,
        year: localDate.year
    });
}

function compareDateRange (startDate, dateToCompare, endDate)
{
    endDate = endDate || startDate;
    return dateToCompare.isSame(startDate, "day") ||
        (dateToCompare.isAfter(startDate, "day") && dateToCompare.isBefore(endDate, "day")) ||
        dateToCompare.isSame(endDate, "day");
}

function createRecurrence (recurrence, startDate)
{
    var unit = "days";
    var interval = 1;

    if (!recurrence)
    {
        return null;
    }

    interval = recurrence.interval || interval;

    switch (recurrence.frequency)
    {
        case "YEARLY":
            unit = "years";
            break;
        case "WEEKLY":
            unit = "weeks";
            break;
        case "MONTHLY":
            unit = "months";
            break;
    }

    return moment(startDate).recur().every(interval, unit);
}

function getSubExclusionForDate (exclusion, date)
{
    var startDate = exclusion.startDate;
    var endDate = exclusion.endDate || exclusion.startDate;
    var startRecurrence = createRecurrence(exclusion.recurrence, startDate);
    var endRecurrence = createRecurrence(exclusion.recurrence, endDate);
    var diff = endDate.diff(startDate, "days");

    date = moment(date).subtract(1, "days");
    return new Exclusion({
        startDate: startRecurrence.fromDate(moment(date).subtract(diff, "days")).next(1)[0],
        endDate: endRecurrence.fromDate(date).next(1)[0]
    });
}

function makeMomentFromDate (date)
{
    if (typeof date === "object")
    {
        if ("monthValue" in date)
        {
            return localDateToMoment(date);
        }
        else if (date instanceof Array && date.length === 3)
        {
            return localDateToMoment({
                dayOfMonth: date[2],
                monthValue: date[1],
                year: date[0]
            });
        }
    }
    
    return moment(date);
}

function Exclusion (data)
{
    if (!(this instanceof Exclusion))
    {
        return new Exclusion(data);
    }

    _(this).extend(data).value();
    this.startDate = makeMomentFromDate(this.startDate);

    if (this.endDate)
    {
        this.endDate = makeMomentFromDate(this.endDate);
    }

    this.matches = function matches (date)
    {
        date = moment.utc(date);

        /*
         * 28 July 2016: If this date is before exclusion start, don't bother
         * checking any further. This is because moment-recur will generate
         * recurring dates before the specified start time (whereas the library
         * we are using on the recorder does not) so falling back to the recurrence
         * can produce false positives.
         */
        if (date.isBefore(this.startDate, "day"))
        {
            return false;
        }

        if (compareDateRange(this.startDate, date, this.endDate))
        {
            return true;
        }

        if (this.recurrence)
        {
            return getSubExclusionForDate(this, date).matches(date);
        }

        return false;
    };
}

module.exports = Exclusion;
