const _ = require("lodash");
const $ = require("jquery");
const ko = require("knockout");
const moment = require("moment");
const toastr = require("toastr");

const Calendar = require("cattura-ui-library/src/lib/ui/calendar");
const ScheduledEvent = require("cattura-ui-library/src/lib/scheduledEvent");
const Exclusion = require("./exclusion");

function makeTab (options)
{
    const plugin                 = options.plugin;
    const addEditEventDialog     = options.addEditEventDialog;
    const capturecastEventSource = options.capturecastEventSource;
    const apiv1                  = options.apiv1;
    const template               = options.template;
    const settings               = options.settings;
    
    if (addEditEventDialog)
    {
        _(["eventsaved", "recurrencesaved", "eventdeleted"]).forEach(event => {
            addEditEventDialog.addEventListener(event, event => calendar.refresh());
        });
    }
    
    function importCalendar ()
    {
        return plugin.importCalendar().then(events => {
            toastr.success(`${events.length} events imported.`);
            calendar.refresh();
        });
    }

    if (capturecastEventSource)
    {
        capturecastEventSource.addEventListener("SchedulerEventSourceSyncEvent", () => calendar.refresh());
    }

    const TITLE = "Schedule";

    let exclusions = ko.observableArray();
    exclusions.subscribe(() => calendar.recolorDays());

    function eventUpdateCallback (event)
    {
        const newStartTimeMillis = adjustDateForTimezone(event.start.valueOf(), event.timezone, true);
        const newStopTimeMillis = adjustDateForTimezone(event.end.valueOf(), event.timezone, true);
        return apiv1.scheduler.updateEvent({
            id: event.id,
            startTime: newStartTimeMillis,
            duration: (newStopTimeMillis - newStartTimeMillis) / 1000 / 60
        }).then(retval => {
            toastr.success(`Event "${event.title}" saved successfully.`);
            calendar.refresh();
        }).catch(error => {
            toastr.error(error.message || `Event "${event.title}" was not saved successfully.`);
            calendar.refresh();
        });
    }

    function loadExclusions ()
    {
        if (!plugin)
        {
            return Promise.resolve([]);
        }

        return plugin.services.getExclusions().then(dates => {
            exclusions(_(dates).map(Exclusion)
                .sortBy(date => date.startDate.valueOf())
                .value()
            );
        }).catch(console.error);
    }

    let calendar = Calendar({
        timezone: (settings ? settings().timezone : null) || "local",
        header: {
            left: "month,agendaWeek,agendaDay",
            center: "title",
            right: "today prev,next"
        },

        eventDataTransform (event)
        {
            let className = [];
            if (event.status === "FAILED")
            {
                className.push("error");
            }
            else if (event.status === "FUTURE")
            {
                className.push("future");
            }
            else if (event.status === "CAPTURING")
            {
                className.push("current");
            }
            else
            {
                className.push("done");
            }

            return _({
                start: adjustDateForTimezone(event.startTime, event.timezone),
                end: adjustDateForTimezone(event.stopTime, event.timezone),
                className: className,
                editable: event.editable && className === "future"
            }).extend(event).value();
        },

        events (start, end, timezone, callback)
        {
            apiv1.scheduler.getEvents({rangeStart: start.valueOf(), rangeEnd: end.valueOf()}).then(function (events) {
                callback(_(events)
                    .map(ScheduledEvent)
                    .map(event => {
                        if (timezone && moment.tz)
                        {
                            event.timezone = moment.tz.zone(timezone);                    
                        }
                        
                        return event;
                    }).value());
            });
        },

        eventClick (event)
        {
            plugin.addEditEvent(event);
        },

        dayRender (date, cell)
        {
            $(cell).toggleClass("excluded-date", isExcluded(date));
        },

        recolorDays ()
        {
            const t = this;
            this.getElement().find(".fc-day").each(function () {
                const date = moment($(this).attr("data-date"));
                t.dayRender(date, this);
            });
        },

        eventDrop: eventUpdateCallback,
        eventResize: eventUpdateCallback
    });

    function load ()
    {
        loadExclusions();
        calendar.refresh();
    }

    function isExcluded (date)
    {
        date = moment(date);
        return !!(_(exclusions()).find(exclusion => exclusion.matches(date)));
    }

    function adjustDateForTimezone (timestamp, timezone, isAdjustingOutOfZone)
    {
        if (!timezone)
        {
            return timestamp;
        }
        
        return timestamp - ((timezone.utcOffset(timestamp) * 1000 * 60) * (isAdjustingOutOfZone ? -1 : 1));
    }

    return {
        title: TITLE,
        calendar: calendar,
        importCalendar: importCalendar,
        addEditEvent: addEditEventDialog,
        load: load,
        loadExclusions: loadExclusions,
        exclusions: exclusions,
        plugin: plugin,
        render: template
    };
}

module.exports = makeTab;